#include "OgreOcclusionCullingTerrainSceneTraverser.h"
#include "OgreOcclusionCullingTerrainSceneManager.h"

#include <windows.h>

namespace Ogre {
	//-----------------------------------------------------------------------
	OcclusionCullingTerrainSceneTraverser::OcclusionCullingTerrainSceneTraverser(SceneManager *sm, RenderSystem *rsys): 
	OcclusionCullingSceneTraverser(sm, rsys)
	{
	}
	//-----------------------------------------------------------------------
	void OcclusionCullingTerrainSceneTraverser::traverseOctree( Camera *cam, Octree *octree )
	{
		mNumTraversedNodes ++;
	
		if(octree->numNodes() > 0)
		{
			renderOctree(cam, octree);
		}

		for(int i=0; i<8; i++)
		{
			char msg[100];
			sprintf(msg,"current node: %d\n", i);
			OutputDebugString(msg);

			if(octree->mChildren[i & 4][i & 2][i & 1] != 0)
				traverseOctree(cam, octree->mChildren[i & 4][i & 2][i & 1]);			
		}
	}
	//-----------------------------------------------------------------------
	void OcclusionCullingTerrainSceneTraverser::renderScene( Camera *cam, Octree *root )
		//TODO: Renderscene independent from hierarchynode => implement only once
	{
		mNumTraversedNodes = 0;
		mNumQueryCulledNodes = 0;
		mNumFrustumCulledNodes = 0;
		mNumRenderedGeometry = 0;
		
		mDistanceQueue = new OctreePriorityQueue(octreeless<Octree *>(cam));
		mDistanceQueue->push(root);
		mCurrentTestIdx = 0;
		
mCurrentAlgorithm = RENDER_CULL_FRUSTUM;
		switch(mCurrentAlgorithm)
		{
			case RENDER_CULL_FRUSTUM:
				renderCullFrustum(cam);
		        break;
			case RENDER_STOP_AND_WAIT:
				renderStopAndWait(cam);
				break;
			case RENDER_COHERENT:
				renderCoherentWithQueue(cam);
				break;
			default:
				renderCullFrustum(cam);
				break;
		}	
		
		delete mDistanceQueue;

		mFrameId ++;
	}
	//-----------------------------------------------------------------------
	void OcclusionCullingTerrainSceneTraverser::renderCullFrustum(Camera *cam)
	{
		AxisAlignedBox box;
		FrustumPlane plane;

		while(!mDistanceQueue->empty())
		{
			Octree *octree = mDistanceQueue->top();
			mDistanceQueue->pop();
	
			// interesting for visualization purpose
			//TODO: octree->setNodeVisible(false);
			octree->_getCullBounds(&box);

			//if(cam->isVisible(box, &plane))
			//{
				// update node's visited flag 
				//TODO: octree->setLastVisited(mFrameId);
				//TODO: octree->setNodeVisible(true);
				traverseOctree(cam, octree);
			/*}
			else
			{			
				mNumFrustumCulledNodes ++;
			}*/
		}
	}	
	
	/** Renders the scene with the hierarchical stop and wait algorithm. */
	void OcclusionCullingTerrainSceneTraverser::renderStopAndWait( Camera *cam )
	{
	}

	/** Renders the scene with the coherent hierarchical algorithm and the query queye. */
	void OcclusionCullingTerrainSceneTraverser::renderCoherentWithQueue( Camera *cam )
	{
	}
	
	//-----------------------------------------------------------------------
/*	bool OcclusionCullingTerrainSceneTraverser::isLeaf( SceneNode *node )
	{
		return (node->numChildren() == 0);
	}*/
	//-----------------------------------------------------------------------
	void OcclusionCullingTerrainSceneTraverser::pullUpVisibility( Octree *node )
	{
	}
	//-----------------------------------------------------------------------
	void OcclusionCullingTerrainSceneTraverser::renderOctree( Camera *cam, Octree *octree )
	{
		setRenderingMode(MODE_RENDER);
		((OcclusionCullingTerrainSceneManager *)mSceneManager)->_renderOctant(cam, octree);
	}
}	
